; Control a small stepper motor with a PIC12C508
;
; There are two controls : STEP and DIRECTION.
; The motor steps when an impulse is received by the STEP input.
; If the DIRECTION button is pressed the direction of rotation is reversed.

         PROCESSOR       12C508
         RADIX           HEX
         INCLUDE         "P12C508.INC"

         __FUSES _MCLRE_OFF & _CP_OFF & _WDT_OFF & _IntRC_OSC

OPMASK   EQU     B'11000000'           ;
BMASK    EQU     B'00101000'           ;All bits output (except GP3 and 5)
STEP_BTN EQU     03                    ;GP3 = Step
DIR_BTN  EQU     05                    ;GP5 = Direction
DELAY1   EQU     0C
DELAY2   EQU     0D
INDEX    EQU     0E                    ;Step index

         ORG 0                         ;Start here

         MOVWF   OSCCAL                ;Setting options
         MOVLW   OPMASK                ;
         OPTION                        ;
         MOVLW   BMASK                 ;
         TRIS    GPIO                  ;

         CLRF    INDEX                 ;Initialize motor
         MOVFW   INDEX                 ;
         CALL    STEP                  ;
         MOVWF   GPIO                  ;

NEXT     BTFSS   GPIO,STEP_BTN         ;Aspetta che l'input sia alto
         GOTO    NEXT                  ;

         BTFSS   GPIO,DIR_BTN          ;
         GOTO    CWISE                 ;
         INCF    INDEX,W               ;
         GOTO    NEW_IDX               ;
CWISE    DECF    INDEX,W               ;
NEW_IDX  ANDLW   .7                    ;
         MOVWF   INDEX                 ;

         CALL    STEP                  ;Convert the index into a bit pattern
         MOVWF   GPIO                  ;

WAIT0 	 BTFSC   GPIO,STEP_BTN         ;Aspetta che l'input torni a zero
         GOTO    WAIT0                 ;

         GOTO    NEXT                  ;
	
STEP     ADDWF   PCL,F                 ;Routine to get step index
         RETLW   B'00000001'           ;
         RETLW   B'00000101'           ;
         RETLW   B'00000100'           ;
         RETLW   B'00000110'           ;
         RETLW   B'00000010'           ;
         RETLW   B'00010010'           ;
         RETLW   B'00010000'           ;
         RETLW   B'00010001'           ;
	
         END
